import logging
import os
import shlex
import subprocess
import time

from barix import HTTPRequestsAPI

from .constants import *

log = logging.getLogger('flask-backend')

def restartServicesFromFile():
    #global readyState
    # check if file already exists
    if os.path.isfile(RESTART_SERVICES_FILE_PATH):
        finished = False
        while not finished:
            try:
                proc = None
                # read services from file
                servicesList = readFromRestartServicesFile()
                if len(servicesList) > 0:
                    log.info('List of services to restart: {}'.format(servicesList))
                    if "reboot" in servicesList:
                        cmd = "reboot"
                        log.info("Executing: {}".format(cmd))
                        proc = subprocess.Popen(shlex.split(cmd), shell=False)
                        proc.wait()
                        finished = True
                    elif "backend_flask" in servicesList:
                        log.info("Restarting backend")
                        servicesList.remove("backend_flask")
                        writeIntoRestartServicesFile(servicesList)
                        r = HTTPRequestsAPI.post(
                            'http://{}:{}{}'.format('127.0.0.1', 50500, '/api/backend/restart'), {})
                        if r.status != 201:
                            log.error("Restarting backend failed!")
                            # something went wrong
                            #servicesList.append("backend_flask")
                            #writeIntoRestartServicesFile(servicesList)
                            #TODO: raise an exeception (which one?)
                    else:
                        path = "/etc/rc5.d/"
                        # restart services by order
                        sortedServicesList = sorted(servicesList)
                        # log.debug('Sorted list of services to restart: {}'.format(sortedServicesList))
                        service = sortedServicesList.pop(0)
                        if service == "epic.main.volume" or service == "application.audio.volume":
                            vol = None
                            from .uci import getValueOfUci
                            if service == "epic.main.volume":
                                vol = getValueOfUci("epic", "main", "volume")
                            if service == "application.audio.volume":
                                vol = getValueOfUci("application", "audio", "volume")
                            mixer = "Line Out"
                            try:
                                f = open("/etc/barix/soundcard.conf", 'r')
                                lines = f.read().split('\n')
                                for line in lines:
                                    if line.startswith('master.control'):
                                        mixer = ((line.split('='))[1]).strip('\n')
                                f.close()
                            except Exception as e:
                                log.error("An error occurred while getting mixer: {}. Going to use mixer {}.".format(e, mixer))
                            cmd = "/usr/bin/amixer -q -c 0 set '" + mixer + "' " + vol + "%"
                            log.info("Executing: {}".format(cmd))
                            proc = subprocess.Popen(shlex.split(cmd), shell=False)

                        elif service == "restart_main_app":
                            cmd = "barix-wd --pid-file=/var/run/msx00-app.pid --restart"
                            log.info("Executing: {}".format(cmd))
                            proc = subprocess.Popen(shlex.split(cmd), shell=False)

                        else:
                            cmd = path + service + " restart"
                            log.info("Executing: {}".format(cmd))
                            proc = subprocess.Popen(shlex.split(cmd), shell=False)

                        # delete service from file
                        writeIntoRestartServicesFile(sortedServicesList)
                        finished = True
                        if proc is not None:
                            proc.wait()
                            if proc.returncode != 0:
                                log.error("Error trying to restart service {}:{}".format(service, proc.returncode))
                            else:
                                log.debug("Service {} restarted with success.".format(service))
                else:
                    log.info("No services to restart: file {} is empty".format(RESTART_SERVICES_FILE_PATH))
                    finished = True
            except Exception as e:
                log.error("An error occurred while restarting services: {}".format(e))
                finished = True
                raise e
            time.sleep(1)
    else:
        log.info("No services to restart: file {} does not exist".format(RESTART_SERVICES_FILE_PATH))

def readFromRestartServicesFile():
    try:
        servicesFromFile = []
        if os.path.isfile(RESTART_SERVICES_FILE_PATH):
            f = open(RESTART_SERVICES_FILE_PATH, 'r')
            content = f.readlines()
            for elem in content:
                servicesFromFile.append(elem.strip('\n'))
            # log.debug('file contents {}'.format(content))
            # if content != '':
            # servicesFromFile = ast.literal_eval(content)
    except Exception as e:
        log.error("An error occured while reading from file {}:{}".format(RESTART_SERVICES_FILE_PATH,e))
        raise e
    else:
        return servicesFromFile

def writeIntoRestartServicesFile(listOfServices):
    try:
        strListOfServices = ""
        for service in listOfServices:
            strListOfServices = strListOfServices + service + "\n"
        #log.debug(strListOfServices)
        strListOfServices = strListOfServices[:-1] #remove last \n
        log.debug("Writting into file {}: {}".format(RESTART_SERVICES_FILE_PATH, strListOfServices))
        f = open(RESTART_SERVICES_FILE_PATH, 'w')
        f.write(strListOfServices)
        f.close()
    except Exception as e:
        log.error("Error writing into {}:{}".format(RESTART_SERVICES_FILE_PATH, e))
        raise e